# matrix_values.txt list line by line each of the 1050 values of the all 42 5x5
# matrices. These are explicit symbolic expressions in terms the parameters and
# a 12th root of unity, without powers of t

# tight_weight.txt contains the powers of t, which are the same for each of the
# 42 matrices. This file is a list of 25 integers.

# values.txt contains the values of the 36 parameters the decomposition is given
# in terms of 

# Here is example sage code to parse the above files

F = CyclotomicField(12)
R = LaurentPolynomialRing(F,'z',36)

subs = {str(e) : e for e in R.gens()}
subs['zeta12'] = F.gen()
matrix_values = list([R(sage_eval(e,locals=subs)) for e in open('matrix_entries.txt')])
z = list(map(ComplexField(100),open('values.txt')))
tight_weight = list(map(int,open('tight_weight.txt')))

# Here are the relevant tensors, given as dictionaries mapping support to value
Tskewcw4 = {(0, 1, 3): 1,
     (0, 2, 4): 1,
     (0, 3, 1): -1,
     (0, 4, 2): -1,
     (1, 0, 3): -1,
     (1, 3, 0): 1,
     (2, 0, 4): -1,
     (2, 4, 0): 1,
     (3, 0, 1): 1,
     (3, 1, 0): -1,
     (4, 0, 2): 1,
     (4, 2, 0): -1}

# kronecker square of the above
Tskewcw4_squared = { (i1*5+i2,j1*5+j2,k1*5+k2) : e1*e2 
        for (i1,j1,k1),e1 in Tskewcw4.items() for (i2,j2,k2),e2 in Tskewcw4.items() }

# The claim of the paper can be checked with this function
def check_paper():
    matrix_values_subbed = [e(z) for e in matrix_values]
    zeros = []
    from itertools import combinations_with_replacement
    for i,j,k in combinations_with_replacement(range(25),3):
        if (i,j,k) in Tskewcw4_squared:
            assert tight_weight[i] + tight_weight[j] + tight_weight[k] == 0
        if tight_weight[i] + tight_weight[j] + tight_weight[k] <= 0:
            zero = sum(matrix_values_subbed[25*s+i]
                    * matrix_values_subbed[25*s+j]
                    * matrix_values_subbed[25*s+k] for s in range(42)) \
                - Tskewcw4_squared.get((i,j,k),0)
            zeros.append(zero)
    worst_violation = max(map(abs,zeros))

    print('The maximum equation violation is {}\n'.format(worst_violation))

check_paper()
